#!/usr/bin/python

# Copyright 2008 Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests that properties conditioned on more than one other property work as
# expected.

import BoostBuild


###############################################################################
#
# test_multiple_conditions()
# --------------------------
#
###############################################################################

def test_multiple_conditions():
    """Basic tests for properties conditioned on multiple other properties."""

    t = BoostBuild.Tester(["--user-config=", "--ignore-site-config",
        "toolset=testToolset"], pass_toolset=False, use_test_config=False)

    t.write("testToolset.jam", """\
import feature ;
feature.extend toolset : testToolset ;
rule init ( ) { }
""")

    t.write("testToolset.py", """\
from b2.build import feature
feature.extend('toolset', ["testToolset"])
def init ( ): pass
""")

    t.write("jamroot.jam", """\
import feature ;
import notfile ;
import toolset ;

feature.feature description : : free incidental ;
feature.feature aaa : 1 0 : incidental ;
feature.feature bbb : 1 0 : incidental ;
feature.feature ccc : 1 0 : incidental ;

rule buildRule ( name : targets ? : properties * )
{
    for local description in [ feature.get-values description : $(properties) ]
    {
        ECHO "description:" /$(description)/ ;
    }
}

notfile testTarget1 : @buildRule : :
    <description>d
    <aaa>0:<description>a0
    <aaa>1:<description>a1
    <aaa>0,<bbb>0:<description>a0-b0
    <aaa>0,<bbb>1:<description>a0-b1
    <aaa>1,<bbb>0:<description>a1-b0
    <aaa>1,<bbb>1:<description>a1-b1
    <aaa>0,<bbb>0,<ccc>0:<description>a0-b0-c0
    <aaa>0,<bbb>0,<ccc>1:<description>a0-b0-c1
    <aaa>0,<bbb>1,<ccc>1:<description>a0-b1-c1
    <aaa>1,<bbb>0,<ccc>1:<description>a1-b0-c1
    <aaa>1,<bbb>1,<ccc>0:<description>a1-b1-c0
    <aaa>1,<bbb>1,<ccc>1:<description>a1-b1-c1 ;
""")

    t.run_build_system(["aaa=1", "bbb=1", "ccc=1"])
    t.expect_output_lines("description: /d/"              )
    t.expect_output_lines("description: /a0/"      , False)
    t.expect_output_lines("description: /a1/"             )
    t.expect_output_lines("description: /a0-b0/"   , False)
    t.expect_output_lines("description: /a0-b1/"   , False)
    t.expect_output_lines("description: /a1-b0/"   , False)
    t.expect_output_lines("description: /a1-b1/"          )
    t.expect_output_lines("description: /a0-b0-c0/", False)
    t.expect_output_lines("description: /a0-b0-c1/", False)
    t.expect_output_lines("description: /a0-b1-c1/", False)
    t.expect_output_lines("description: /a1-b0-c1/", False)
    t.expect_output_lines("description: /a1-b1-c0/", False)
    t.expect_output_lines("description: /a1-b1-c1/"       )

    t.run_build_system(["aaa=0", "bbb=0", "ccc=1"])
    t.expect_output_lines("description: /d/"              )
    t.expect_output_lines("description: /a0/"             )
    t.expect_output_lines("description: /a1/"      , False)
    t.expect_output_lines("description: /a0-b0/"          )
    t.expect_output_lines("description: /a0-b1/"   , False)
    t.expect_output_lines("description: /a1-b0/"   , False)
    t.expect_output_lines("description: /a1-b1/"   , False)
    t.expect_output_lines("description: /a0-b0-c0/", False)
    t.expect_output_lines("description: /a0-b0-c1/"       )
    t.expect_output_lines("description: /a0-b1-c1/", False)
    t.expect_output_lines("description: /a1-b0-c1/", False)
    t.expect_output_lines("description: /a1-b1-c0/", False)
    t.expect_output_lines("description: /a1-b1-c1/", False)

    t.run_build_system(["aaa=0", "bbb=0", "ccc=0"])
    t.expect_output_lines("description: /d/"              )
    t.expect_output_lines("description: /a0/"             )
    t.expect_output_lines("description: /a1/"      , False)
    t.expect_output_lines("description: /a0-b0/"          )
    t.expect_output_lines("description: /a0-b1/"   , False)
    t.expect_output_lines("description: /a1-b0/"   , False)
    t.expect_output_lines("description: /a1-b1/"   , False)
    t.expect_output_lines("description: /a0-b0-c0/"       )
    t.expect_output_lines("description: /a0-b0-c1/", False)
    t.expect_output_lines("description: /a0-b1-c1/", False)
    t.expect_output_lines("description: /a1-b0-c1/", False)
    t.expect_output_lines("description: /a1-b1-c0/", False)
    t.expect_output_lines("description: /a1-b1-c1/", False)

    t.cleanup()


###############################################################################
#
# test_multiple_conditions_with_toolset_version()
# -----------------------------------------------
#
###############################################################################

def test_multiple_conditions_with_toolset_version():
    """
      Regression tests for properties conditioned on the toolset version
    subfeature and some additional properties.

    """
    toolset = "testToolset" ;

    t = BoostBuild.Tester(["--user-config=", "--ignore-site-config"],
        pass_toolset=False, use_test_config=False)

    t.write(toolset + ".jam", """\
import feature ;
feature.extend toolset : %(toolset)s ;
feature.subfeature toolset %(toolset)s : version : 0 1 ;
rule init ( version ? ) { }
""" % {"toolset": toolset})

    t.write("testToolset.py", """\
from b2.build import feature
feature.extend('toolset', ["%(toolset)s"])
feature.subfeature('toolset', "%(toolset)s", "version", ['0','1'])
def init (version=''): pass
""" % {"toolset": toolset})

    t.write("jamroot.jam", """\
import feature ;
import notfile ;
import toolset ;

toolset.using testToolset ;

feature.feature description : : free incidental ;
feature.feature aaa : 0 1 : incidental ;
feature.feature bbb : 0 1 : incidental ;
feature.feature ccc : 0 1 : incidental ;

rule buildRule ( name : targets ? : properties * )
{
    local ttt = [ feature.get-values toolset                     : $(properties) ] ;
    local vvv = [ feature.get-values toolset-testToolset:version : $(properties) ] ;
    local aaa = [ feature.get-values aaa                         : $(properties) ] ;
    local bbb = [ feature.get-values bbb                         : $(properties) ] ;
    local ccc = [ feature.get-values ccc                         : $(properties) ] ;
    ECHO "toolset:" /$(ttt)/ "version:" /$(vvv)/ "aaa/bbb/ccc:" /$(aaa)/$(bbb)/$(ccc)/ ;
    for local description in [ feature.get-values description : $(properties) ]
    {
        ECHO "description:" /$(description)/ ;
    }
}

notfile testTarget1 : @buildRule : :
    <toolset>testToolset,<aaa>0:<description>t-a0
    <toolset>testToolset,<aaa>1:<description>t-a1

    <toolset>testToolset-0,<aaa>0:<description>t0-a0
    <toolset>testToolset-0,<aaa>1:<description>t0-a1
    <toolset>testToolset-1,<aaa>0:<description>t1-a0
    <toolset>testToolset-1,<aaa>1:<description>t1-a1

    <toolset>testToolset,<aaa>0,<bbb>0:<description>t-a0-b0
    <toolset>testToolset,<aaa>0,<bbb>1:<description>t-a0-b1
    <toolset>testToolset,<aaa>1,<bbb>0:<description>t-a1-b0
    <toolset>testToolset,<aaa>1,<bbb>1:<description>t-a1-b1

    <aaa>0,<toolset>testToolset,<bbb>0:<description>a0-t-b0
    <aaa>0,<toolset>testToolset,<bbb>1:<description>a0-t-b1
    <aaa>1,<toolset>testToolset,<bbb>0:<description>a1-t-b0
    <aaa>1,<toolset>testToolset,<bbb>1:<description>a1-t-b1

    <aaa>0,<bbb>0,<toolset>testToolset:<description>a0-b0-t
    <aaa>0,<bbb>1,<toolset>testToolset:<description>a0-b1-t
    <aaa>1,<bbb>0,<toolset>testToolset:<description>a1-b0-t
    <aaa>1,<bbb>1,<toolset>testToolset:<description>a1-b1-t

    <toolset>testToolset-0,<aaa>0,<bbb>0:<description>t0-a0-b0
    <toolset>testToolset-0,<aaa>0,<bbb>1:<description>t0-a0-b1
    <toolset>testToolset-0,<aaa>1,<bbb>0:<description>t0-a1-b0
    <toolset>testToolset-0,<aaa>1,<bbb>1:<description>t0-a1-b1
    <toolset>testToolset-1,<aaa>0,<bbb>0:<description>t1-a0-b0
    <toolset>testToolset-1,<aaa>0,<bbb>1:<description>t1-a0-b1
    <toolset>testToolset-1,<aaa>1,<bbb>0:<description>t1-a1-b0
    <toolset>testToolset-1,<aaa>1,<bbb>1:<description>t1-a1-b1

    <aaa>0,<toolset>testToolset-1,<bbb>0:<description>a0-t1-b0
    <aaa>0,<toolset>testToolset-1,<bbb>1:<description>a0-t1-b1
    <aaa>1,<toolset>testToolset-0,<bbb>0:<description>a1-t0-b0
    <aaa>1,<toolset>testToolset-0,<bbb>1:<description>a1-t0-b1

    <bbb>0,<aaa>1,<toolset>testToolset-0:<description>b0-a1-t0
    <bbb>0,<aaa>0,<toolset>testToolset-1:<description>b0-a0-t1
    <bbb>0,<aaa>1,<toolset>testToolset-1:<description>b0-a1-t1
    <bbb>1,<aaa>0,<toolset>testToolset-1:<description>b1-a0-t1
    <bbb>1,<aaa>1,<toolset>testToolset-0:<description>b1-a1-t0
    <bbb>1,<aaa>1,<toolset>testToolset-1:<description>b1-a1-t1 ;
""")

    t.run_build_system(["aaa=1", "bbb=1", "ccc=1", "toolset=%s-0" % toolset])
    t.expect_output_lines("description: /t-a0/"    , False)
    t.expect_output_lines("description: /t-a1/"           )
    t.expect_output_lines("description: /t0-a0/"   , False)
    t.expect_output_lines("description: /t0-a1/"          )
    t.expect_output_lines("description: /t1-a0/"   , False)
    t.expect_output_lines("description: /t1-a1/"   , False)
    t.expect_output_lines("description: /t-a0-b0/" , False)
    t.expect_output_lines("description: /t-a0-b1/" , False)
    t.expect_output_lines("description: /t-a1-b0/" , False)
    t.expect_output_lines("description: /t-a1-b1/"        )
    t.expect_output_lines("description: /a0-t-b0/" , False)
    t.expect_output_lines("description: /a0-t-b1/" , False)
    t.expect_output_lines("description: /a1-t-b0/" , False)
    t.expect_output_lines("description: /a1-t-b1/"        )
    t.expect_output_lines("description: /a0-b0-t/" , False)
    t.expect_output_lines("description: /a0-b1-t/" , False)
    t.expect_output_lines("description: /a1-b0-t/" , False)
    t.expect_output_lines("description: /a1-b1-t/"        )
    t.expect_output_lines("description: /t0-a0-b0/", False)
    t.expect_output_lines("description: /t0-a0-b1/", False)
    t.expect_output_lines("description: /t0-a1-b0/", False)
    t.expect_output_lines("description: /t0-a1-b1/"       )
    t.expect_output_lines("description: /t1-a0-b0/", False)
    t.expect_output_lines("description: /t1-a0-b1/", False)
    t.expect_output_lines("description: /t1-a1-b0/", False)
    t.expect_output_lines("description: /t1-a1-b1/", False)
    t.expect_output_lines("description: /a0-t1-b0/", False)
    t.expect_output_lines("description: /a0-t1-b1/", False)
    t.expect_output_lines("description: /a1-t0-b0/", False)
    t.expect_output_lines("description: /a1-t0-b1/"       )
    t.expect_output_lines("description: /b0-a1-t0/", False)
    t.expect_output_lines("description: /b0-a0-t1/", False)
    t.expect_output_lines("description: /b0-a1-t1/", False)
    t.expect_output_lines("description: /b1-a0-t1/", False)
    t.expect_output_lines("description: /b1-a1-t0/"       )
    t.expect_output_lines("description: /b1-a1-t1/", False)

    t.run_build_system(["aaa=1", "bbb=1", "ccc=1", "toolset=%s-1" % toolset])
    t.expect_output_lines("description: /t-a0/"    , False)
    t.expect_output_lines("description: /t-a1/"           )
    t.expect_output_lines("description: /t0-a0/"   , False)
    t.expect_output_lines("description: /t0-a1/"   , False)
    t.expect_output_lines("description: /t1-a0/"   , False)
    t.expect_output_lines("description: /t1-a1/"          )
    t.expect_output_lines("description: /t-a0-b0/" , False)
    t.expect_output_lines("description: /t-a0-b1/" , False)
    t.expect_output_lines("description: /t-a1-b0/" , False)
    t.expect_output_lines("description: /t-a1-b1/"        )
    t.expect_output_lines("description: /a0-t-b0/" , False)
    t.expect_output_lines("description: /a0-t-b1/" , False)
    t.expect_output_lines("description: /a1-t-b0/" , False)
    t.expect_output_lines("description: /a1-t-b1/"        )
    t.expect_output_lines("description: /a0-b0-t/" , False)
    t.expect_output_lines("description: /a0-b1-t/" , False)
    t.expect_output_lines("description: /a1-b0-t/" , False)
    t.expect_output_lines("description: /a1-b1-t/"        )
    t.expect_output_lines("description: /t0-a0-b0/", False)
    t.expect_output_lines("description: /t0-a0-b1/", False)
    t.expect_output_lines("description: /t0-a1-b0/", False)
    t.expect_output_lines("description: /t0-a1-b1/", False)
    t.expect_output_lines("description: /t1-a0-b0/", False)
    t.expect_output_lines("description: /t1-a0-b1/", False)
    t.expect_output_lines("description: /t1-a1-b0/", False)
    t.expect_output_lines("description: /t1-a1-b1/"       )
    t.expect_output_lines("description: /a0-t1-b0/", False)
    t.expect_output_lines("description: /a0-t1-b1/", False)
    t.expect_output_lines("description: /a1-t0-b0/", False)
    t.expect_output_lines("description: /a1-t0-b1/", False)
    t.expect_output_lines("description: /b0-a1-t0/", False)
    t.expect_output_lines("description: /b0-a0-t1/", False)
    t.expect_output_lines("description: /b0-a1-t1/", False)
    t.expect_output_lines("description: /b1-a0-t1/", False)
    t.expect_output_lines("description: /b1-a1-t0/", False)
    t.expect_output_lines("description: /b1-a1-t1/"       )

    t.cleanup()


###############################################################################
#
# main()
# ------
#
###############################################################################

test_multiple_conditions()
test_multiple_conditions_with_toolset_version()



#!/usr/bin/python

# Copyright 2003 Dave Abrahams
# Copyright 2006 Rene Rivera
# Copyright 2003 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Demonstration that module variables have the correct effect in actions.

import BoostBuild
import os
import re

t = BoostBuild.Tester(["-d+1"], pass_toolset=0)

t.write("boost-build.jam", "boost-build . ;")
t.write("bootstrap.jam", """\
# Top-level rule causing a target to be built by invoking the specified action.
rule make ( target : sources * : act )
{
    DEPENDS all : $(target) ;
    DEPENDS $(target) : $(sources) ;
    $(act) $(target) : $(sources) ;
}

X1 = X1-global ;
X2 = X2-global ;
X3 = X3-global ;

module A
{
    X1 = X1-A ;

    rule act ( target )
    {
        NOTFILE $(target) ;
        ALWAYS $(target) ;
    }

    actions act { echo A.act $(<): $(X1) $(X2) $(X3) }

    make t1 : : A.act ;
    make t2 : : A.act ;
    make t3 : : A.act ;
}

module B
{
    X2 = X2-B ;

    actions act { echo B.act $(<): $(X1) $(X2) $(X3) }

    make t1 : : B.act ;
    make t2 : : B.act ;
    make t3 : : B.act ;
}

actions act { echo act $(<): $(X1) $(X2) $(X3) }

make t1 : : act ;
make t2 : : act ;
make t3 : : act ;

X1 on t1 = X1-t1 ;
X2 on t2 = X2-t2 ;
X3 on t3 = X3-t3 ;

DEPENDS all : t1 t2 t3 ;
""")

expected_lines = [
    "...found 4 targets...",
    "...updating 3 targets...",
    "A.act t1",
    "A.act t1: X1-t1   ",
    "B.act t1",
    "B.act t1: X1-t1 X2-B  ",
    "act t1",
    "act t1: X1-t1 X2-global X3-global ",
    "A.act t2",
    "A.act t2: X1-A X2-t2  ",
    "B.act t2",
    "B.act t2:  X2-t2  ",
    "act t2",
    "act t2: X1-global X2-t2 X3-global ",
    "A.act t3",
    "A.act t3: X1-A  X3-t3 ",
    "B.act t3",
    "B.act t3:  X2-B X3-t3 ",
    "act t3",
    "act t3: X1-global X2-global X3-t3 ",
    "...updated 3 targets...",
    ""]

# Accommodate for the fact that on Unixes, a call to 'echo 1   2     3  '
# produces '1 2 3' (note the spacing).
if os.name != 'nt':
    expected_lines = [re.sub("  +", " ", x.rstrip()) for x in expected_lines]

t.run_build_system()
t.expect_output_lines(expected_lines)
t.expect_nothing_more()
t.cleanup()



#!/usr/bin/python

# Copyright (C) 2003. Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Test the 'glob' rule in Jamfile context.

import BoostBuild


def test_basic():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "")
    t.write("d1/a.cpp", "int main() {}\n")
    t.write("d1/jamfile.jam", "exe a : [ glob *.cpp ] ../d2/d//l ;")
    t.write("d2/d/l.cpp", """\
#if defined(_WIN32)
__declspec(dllexport)
void force_import_lib_creation() {}
#endif
""")
    t.write("d2/d/jamfile.jam", "lib l : [ glob *.cpp ] ;")
    t.write("d3/d/jamfile.jam", "exe a : [ glob ../*.cpp ] ;")
    t.write("d3/a.cpp", "int main() {}\n")

    t.run_build_system(subdir="d1")
    t.expect_addition("d1/bin/$toolset/debug*/a.exe")

    t.run_build_system(subdir="d3/d")
    t.expect_addition("d3/d/bin/$toolset/debug*/a.exe")

    t.rm("d2/d/bin")
    t.run_build_system(subdir="d2/d")
    t.expect_addition("d2/d/bin/$toolset/debug*/l.dll")

    t.cleanup()


def test_source_location():
    """
      Test that when 'source-location' is explicitly-specified glob works
    relative to the source location.

    """
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "")
    t.write("d1/a.cpp", "very bad non-compilable file\n")
    t.write("d1/src/a.cpp", "int main() {}\n")
    t.write("d1/jamfile.jam", """\
project : source-location src ;
exe a : [ glob *.cpp ] ../d2/d//l ;
""")
    t.write("d2/d/l.cpp", """\
#if defined(_WIN32)
__declspec(dllexport)
void force_import_lib_creation() {}
#endif
""")
    t.write("d2/d/jamfile.jam", "lib l : [ glob *.cpp ] ;")

    t.run_build_system(subdir="d1")
    t.expect_addition("d1/bin/$toolset/debug*/a.exe")

    t.cleanup()


def test_wildcards_and_exclusion_patterns():
    """
        Test that wildcards can include directories. Also test exclusion
     patterns.

    """
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "")
    t.write("d1/src/foo/a.cpp", "void bar(); int main() { bar(); }\n")
    t.write("d1/src/bar/b.cpp", "void bar() {}\n")
    t.write("d1/src/bar/bad.cpp", "very bad non-compilable file\n")
    t.write("d1/jamfile.jam", """\
project : source-location src ;
exe a : [ glob foo/*.cpp bar/*.cpp : bar/bad* ] ../d2/d//l ;
""")
    t.write("d2/d/l.cpp", """\
#if defined(_WIN32)
__declspec(dllexport)
void force_import_lib_creation() {}
#endif
""")
    t.write("d2/d/jamfile.jam", "lib l : [ glob *.cpp ] ;")

    t.run_build_system(subdir="d1")
    t.expect_addition("d1/bin/$toolset/debug*/a.exe")

    t.cleanup()


def test_glob_tree():
    """Test that 'glob-tree' works."""

    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "")
    t.write("d1/src/foo/a.cpp", "void bar(); int main() { bar(); }\n")
    t.write("d1/src/bar/b.cpp", "void bar() {}\n")
    t.write("d1/src/bar/bad.cpp", "very bad non-compilable file\n")
    t.write("d1/jamfile.jam", """\
project : source-location src ;
exe a : [ glob-tree *.cpp : bad* ] ../d2/d//l ;
""")
    t.write("d2/d/l.cpp", """\
#if defined(_WIN32)
__declspec(dllexport)
void force_import_lib_creation() {}
#endif
""")
    t.write("d2/d/jamfile.jam", "lib l : [ glob *.cpp ] ;")

    t.run_build_system(subdir="d1")
    t.expect_addition("d1/bin/$toolset/debug*/a.exe")

    t.cleanup()


def test_directory_names_in_glob_tree():
    """Test that directory names in patterns for 'glob-tree' are rejected."""

    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "")
    t.write("d1/src/a.cpp", "very bad non-compilable file\n")
    t.write("d1/src/foo/a.cpp", "void bar(); int main() { bar(); }\n")
    t.write("d1/src/bar/b.cpp", "void bar() {}\n")
    t.write("d1/src/bar/bad.cpp", "very bad non-compilable file\n")
    t.write("d1/jamfile.jam", """\
project : source-location src ;
exe a : [ glob-tree foo/*.cpp bar/*.cpp : bad* ] ../d2/d//l ;
""")
    t.write("d2/d/l.cpp", """\
#if defined(_WIN32)
__declspec(dllexport)
void force_import_lib_creation() {}
#endif
""")
    t.write("d2/d/jamfile.jam", "lib l : [ glob *.cpp ] ;")

    t.run_build_system(subdir="d1", status=1)
    t.expect_output_lines("error: The patterns * may not include directory")

    t.cleanup()


def test_glob_with_absolute_names():
    """Test that 'glob' works with absolute names."""

    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "")
    t.write("d1/src/a.cpp", "very bad non-compilable file\n")
    t.write("d1/src/foo/a.cpp", "void bar(); int main() { bar(); }\n")
    t.write("d1/src/bar/b.cpp", "void bar() {}\n")
    # Note that to get the current dir, we use bjam's PWD, not Python's
    # os.getcwd(), because the former will always return a long path while the
    # latter might return a short path, which would confuse path.glob.
    t.write("d1/jamfile.jam", """\
project : source-location src ;
local pwd = [ PWD ] ;  # Always absolute.
exe a : [ glob $(pwd)/src/foo/*.cpp $(pwd)/src/bar/*.cpp ] ../d2/d//l ;
""")
    t.write("d2/d/l.cpp", """\
#if defined(_WIN32)
__declspec(dllexport)
void force_import_lib_creation() {}
#endif
""")
    t.write("d2/d/jamfile.jam", "lib l : [ glob *.cpp ] ;")

    t.run_build_system(subdir="d1")
    t.expect_addition("d1/bin/$toolset/debug*/a.exe")

    t.cleanup()


def test_glob_excludes_in_subdirectory():
    """
      Regression test: glob excludes used to be broken when building from a
    subdirectory.

    """
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", "build-project p ;")
    t.write("p/p.c", "int main() {}\n")
    t.write("p/p_x.c", "very bad non-compilable file\n")
    t.write("p/jamfile.jam", "exe p : [ glob *.c : p_x.c ] ;")

    t.run_build_system(subdir="p")
    t.expect_addition("p/bin/$toolset/debug*/p.exe")

    t.cleanup()


test_basic()
test_source_location()
test_wildcards_and_exclusion_patterns()
test_glob_tree()
test_directory_names_in_glob_tree()
test_glob_with_absolute_names()
test_glob_excludes_in_subdirectory()



# Status: ported.
# Base revison: 64488
#
# Copyright (c) 2010 Vladimir Prus.
#
# Use, modification and distribution is subject to the Boost Software
# License Version 1.0. (See accompanying file LICENSE_1_0.txt or
# http://www.boost.org/LICENSE_1_0.txt)

# This module defines function to help with two main tasks:
#
# - Discovering build-time configuration for the purposes of adjusting
#   build process.
# - Reporting what is built, and how it is configured.

import b2.build.property as property
import b2.build.property_set as property_set

from b2.build import targets as targets_

from b2.manager import get_manager
from b2.util.sequence import unique
from b2.util import bjam_signature, value_to_jam, is_iterable

import bjam
import os

__width = 30

def set_width(width):
    global __width
    __width = 30

__components = []
__built_components = []
__component_logs = {}
__announced_checks = False

__log_file = None
__log_fd = -1

def register_components(components):
    """Declare that the components specified by the parameter exist."""
    assert is_iterable(components)
    __components.extend(components)

def components_building(components):
    """Declare that the components specified by the parameters will be build."""
    assert is_iterable(components)
    __built_components.extend(components)

def log_component_configuration(component, message):
    """Report something about component configuration that the user should better know."""
    assert isinstance(component, basestring)
    assert isinstance(message, basestring)
    __component_logs.setdefault(component, []).append(message)

def log_check_result(result):
    assert isinstance(result, basestring)
    global __announced_checks
    if not __announced_checks:
        print "Performing configuration checks"
        __announced_checks = True

    print result

def log_library_search_result(library, result):
    assert isinstance(library, basestring)
    assert isinstance(result, basestring)
    log_check_result(("    - %(library)s : %(result)s" % locals()).rjust(__width))


def print_component_configuration():

    print "\nComponent configuration:"
    for c in __components:
        if c in __built_components:
            s = "building"
        else:
            s = "not building"
        message = "    - %s)" % c
        message = message.rjust(__width)
        message += " : " + s
        for m in __component_logs.get(c, []):
            print "        -" + m
    print ""

__builds_cache = {}

def builds(metatarget_reference, project, ps, what):
    # Attempt to build a metatarget named by 'metatarget-reference'
    # in context of 'project' with properties 'ps'.
    # Returns non-empty value if build is OK.
    assert isinstance(metatarget_reference, basestring)
    assert isinstance(project, targets_.ProjectTarget)
    assert isinstance(ps, property_set.PropertySet)
    assert isinstance(what, basestring)

    result = []

    existing = __builds_cache.get((what, ps), None)
    if existing is None:

        result = False
        __builds_cache[(what, ps)] = False

        targets = targets_.generate_from_reference(
            metatarget_reference, project, ps).targets()
        jam_targets = []
        for t in targets:
            jam_targets.append(t.actualize())

        x = ("    - %s" % what).rjust(__width)
        if bjam.call("UPDATE_NOW", jam_targets, str(__log_fd), "ignore-minus-n"):
            __builds_cache[(what, ps)] = True
            result = True
            log_check_result("%s: yes" % x)
        else:
            log_check_result("%s: no" % x)

        return result
    else:
        return existing

def set_log_file(log_file_name):
    assert isinstance(log_file_name, basestring)
    # Called by Boost.Build startup code to specify name of a file
    # that will receive results of configure checks.  This
    # should never be called by users.
    global __log_file, __log_fd
    dirname = os.path.dirname(log_file_name)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    # Make sure to keep the file around, so that it's not
    # garbage-collected and closed
    __log_file = open(log_file_name, "w")
    __log_fd = __log_file.fileno()

# Frontend rules

class CheckTargetBuildsWorker:

    def __init__(self, target, true_properties, false_properties):
        self.target = target
        self.true_properties = property.create_from_strings(true_properties, True)
        self.false_properties = property.create_from_strings(false_properties, True)

    def check(self, ps):
        assert isinstance(ps, property_set.PropertySet)
        # FIXME: this should not be hardcoded. Other checks might
        # want to consider different set of features as relevant.
        toolset = ps.get('toolset')[0]
        toolset_version_property = "<toolset-" + toolset + ":version>" ;
        relevant = ps.get_properties('target-os') + \
                   ps.get_properties("toolset") + \
                   ps.get_properties(toolset_version_property) + \
                   ps.get_properties("address-model") + \
                   ps.get_properties("architecture")
        rps = property_set.create(relevant)
        t = get_manager().targets().current()
        p = t.project()
        if builds(self.target, p, rps, "%s builds" % self.target):
            choosen = self.true_properties
        else:
            choosen = self.false_properties
        return property.evaluate_conditionals_in_context(choosen, ps)

@bjam_signature((["target"], ["true_properties", "*"], ["false_properties", "*"]))
def check_target_builds(target, true_properties, false_properties):
    worker = CheckTargetBuildsWorker(target, true_properties, false_properties)
    value = value_to_jam(worker.check)
    return "<conditional>" + value

get_manager().projects().add_rule("check-target-builds", check_target_builds)





#!/usr/bin/python

# Copyright 2003 Dave Abrahams
# Copyright 2003 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test the 'symlink' rule.

import os
import BoostBuild


if os.name != 'posix':
    print "The symlink tests can be run on posix only."
    import sys
    sys.exit(1)


t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", "import gcc ;")

t.write("jamfile.jam", """
exe hello : hello.cpp ;
symlink hello_release : hello/<variant>release ;
symlink hello_debug : hello/<variant>debug ;
symlink links/hello_release : hello/<variant>release ;
""")

t.write("hello.cpp", """
int main() {}
""")

t.run_build_system()
t.expect_addition([
    'hello_debug.exe',
    'hello_release.exe',
    'links/hello_release.exe'])

t.cleanup()
